@@ -19,7 +19,6 @@
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  */
 
-const Champlain = imports.gi.Champlain;
 const Gdk = imports.gi.Gdk;
 const Geocode = imports.gi.GeocodeGlib;
 const GObject = imports.gi.GObject;
@@ -27,7 +26,6 @@ const Gtk = imports.gi.Gtk;
 const Mainloop = imports.mainloop;
 
 const Application = imports.application;
-const ExportViewDialog = imports.exportViewDialog;
 const GeocodeFactory = imports.geocode;
 const Location = imports.location;
 const OSMAccountDialog = imports.osmAccountDialog;
@@ -42,9 +40,10 @@ var ContextMenu = GObject.registerClass({
     Template: 'resource:///org/gnome/Maps/ui/context-menu.ui',
     InternalChildren: [ 'whatsHereItem',
                         'geoURIItem',
-                        'exportItem',
                         'addOSMLocationItem',
-                        'routeItem' ],
+                        'routeFromHereItem',
+                        'addIntermediateDestinationItem',
+                        'routeToHereItem' ],
 }, class ContextMenu extends Gtk.Menu {
     _init(params) {
         this._mapView = params.mapView;
@@ -62,15 +61,16 @@ var ContextMenu = GObject.registerClass({
                                     this._onWhatsHereActivated.bind(this));
         this._geoURIItem.connect('activate',
                                  this._onGeoURIActivated.bind(this));
-        this._exportItem.connect('activate',
-                                 this._onExportActivated.bind(this));
         this._addOSMLocationItem.connect('activate',
                                          this._onAddOSMLocationActivated.bind(this));
-        this._routeItem.connect('activate',
-                                this._onRouteActivated.bind(this));
+        this._routeFromHereItem.connect('activate',
+                                        this._onRouteFromHereActivated.bind(this));
+        this._addIntermediateDestinationItem.connect('activate',
+                                        this._onAddIntermediateDestinationActivated.bind(this));
+        this._routeToHereItem.connect('activate',
+                                      this._onRouteToHereActivated.bind(this));
         Application.routeQuery.connect('notify::points',
                                        this._routingUpdate.bind(this));
-        this._routeItem.visible = false;
         this._routingUpdate();
     }
 
@@ -88,36 +88,42 @@ var ContextMenu = GObject.registerClass({
 
     _routingUpdate() {
         let query = Application.routeQuery;
+        let numPoints = query.points.length;
 
-        this._routeItem.sensitive = query.points.length < RouteQuery.MAX_QUERY_POINTS;
-
-        if (query.points.length === 0)
-            return;
-
-        this._routeItem.visible = true;
-        if (!query.points[0].place) {
-            this._routeItem.label = _("Route from here");
-        } else if (query.filledPoints.length > 1) {
-            this._routeItem.label = _("Add destination");
-        } else {
-            this._routeItem.label = _("Route to here");
-        }
+        this._routeFromHereItem.sensitive = numPoints < RouteQuery.MAX_QUERY_POINTS;
+        this._routeToHereItem.sensitive = numPoints < RouteQuery.MAX_QUERY_POINTS;
+        this._addIntermediateDestinationItem.sensitive =
+            query.filledPoints.length >= 2 && numPoints < RouteQuery.MAX_QUERY_POINTS;
     }
 
-    _onRouteActivated() {
+    _onRouteFromHereActivated() {
         let query = Application.routeQuery;
         let location = new Location.Location({ latitude: this._latitude,
                                                longitude: this._longitude,
                                                accuracy: 0 });
         let place = new Place.Place({ location: location });
 
-        if (!query.points[0].place) {
-            query.points[0].place = place;
-        } else if (query.filledPoints.length > 1) {
-            query.addPoint(-1).place = place;
-        } else {
-            query.points[query.points.length - 1].place = place;
-        }
+        query.points[0].place = place;
+    }
+
+    _onRouteToHereActivated() {
+        let query = Application.routeQuery;
+        let location = new Location.Location({ latitude: this._latitude,
+                                               longitude: this._longitude,
+                                               accuracy: 0 });
+        let place = new Place.Place({ location: location });
+
+        query.points.last().place = place;
+    }
+
+    _onAddIntermediateDestinationActivated() {
+        let query = Application.routeQuery;
+        let location = new Location.Location({ latitude: this._latitude,
+                                               longitude: this._longitude,
+                                               accuracy: 0 });
+        let place = new Place.Place({ location: location });
+
+        query.addPoint(-1).place = place;
     }
 
     _onWhatsHereActivated() {
@@ -192,36 +198,4 @@ var ContextMenu = GObject.registerClass({
             }
         });
     }
-
-    _activateExport() {
-        let view = this._mapView.view;
-        let surface = view.to_surface(true);
-        let bbox = view.get_bounding_box();
-        let [latitude, longitude] = bbox.get_center();
-
-        let dialog = new ExportViewDialog.ExportViewDialog({
-            transient_for: this._mainWindow,
-            modal: true,
-            surface: surface,
-            latitude: latitude,
-            longitude: longitude,
-            mapView: this._mapView
-        });
-
-        dialog.connect('response', () => dialog.destroy());
-        dialog.show_all();
-    }
-
-    _onExportActivated() {
-        if (this._mapView.view.state === Champlain.State.DONE) {
-            this._activateExport();
-        } else {
-            let notifyId = this._mapView.view.connect('notify::state', () => {
-                if (this._mapView.view.state === Champlain.State.DONE) {
-                    this._mapView.view.disconnect(notifyId);
-                    this._activateExport();
-                }
-            });
-        }
-    }
 });
